#define screenWidthTexelSize	rcpres[0]
#define screenHeightTexelSize	rcpres[1]

extern float4 screenParams = { 1280.0f, 1024.0f, 0.0f, 0.0f };
extern float2 rcpres[2];	// render target X and Y

extern float speed <
    string UIName = "speed";
    string UIWidget = "slider";
    float UIMin = -1.0f;
    float UIMax = 2.0f;
    float UIStep = 0.01f;
>;

extern float horizon <
    string UIName = "horizon";
    string UIWidget = "slider";
    float UIMin = -1.0f;
    float UIMax = 2.0f;
    float UIStep = 0.01f;
>;

extern float cx <
    string UIName = "center X";
    string UIWidget = "slider";
    float UIMin = -1.0f;
    float UIMax = 2.0f;
    float UIStep = 0.01f;
>;

static float2 Center = { 1 - cx, 1 - horizon };

#define numBlurSamples 16

static float blurCutoff = 0.1f;
static float blurScale = 0.007f;
static float gaussianStdDev = 6.0f;
static float blurOffsetMax = .8f;
static float gaussianSum = 1.0f;


texture thisframe;	// the current frame from before any effects are applied from this file


sampler currentFrameSampler = sampler_state
{
	texture = <thisframe>;
	AddressU = CLAMP;
	AddressV = CLAMP;
	MINFILTER = POINT;
	MAGFILTER = POINT;
};

struct VSOUT
{
	float4 vertPos : POSITION;
	float2 UVCoord : TEXCOORD0;
};

struct VSIN
{
	float4 vertPos : POSITION0;
	float2 UVCoord : TEXCOORD0;
};

VSOUT FrameVS(VSIN IN)
{
	VSOUT OUT = (VSOUT)0.0f;	// initialize to zero, avoid complaints.
	OUT.vertPos = IN.vertPos;
	OUT.UVCoord = IN.UVCoord;
	return OUT;
}

half4 ApplyRadialBlur(float2 UVCoord : TEXCOORD0) : COLOR0
{

  float2 texelSize = (float2)rcpres;
  float2 offsetBlurDirection = UVCoord + texelSize * 0.5 - Center;
  
  
  float2 mags = offsetBlurDirection * offsetBlurDirection;
  float blurmag = sqrt(mags.x + mags.y);
  blurmag *= blurmag * .2;
  
  //blurmag = clamp(blurmag, 0, .2);
  //return half4((1 + blurmag) * speed, 0, 0, 1);
        
  half3 blurredColor = 0;
  gaussianSum = 0.0f;
  
  for(float i=0; i<numBlurSamples; i++) {
   	float scale = 1 + -(speed * blurmag) * ( i / (float)(numBlurSamples - 1) );
    half4 current = tex2D( currentFrameSampler, offsetBlurDirection.xy * scale + Center );
    float weight = exp((i*i) / (2.0f * gaussianStdDev * gaussianStdDev)) / gaussianStdDev;
	  gaussianSum += weight;
  	blurredColor += current.rgb * weight;
 	}
 	blurredColor /= gaussianSum;
    
  return half4(blurredColor, 1); 
}

technique t0 
{
	pass p0
	{
		//VertexShader = compile vs_1_1 FrameVS();	// unneeded unless the compile target is PS3.0
		PixelShader = compile ps_2_0 ApplyRadialBlur();
		CullMode = none;
		ZEnable = false;
		ZWriteEnable = false;
		AlphaBlendEnable = false;
	}
}